unit DPRegstr;

{$I JCL.INC}

{*********************************************************************}
{ Programmer:  Kevin S Gallagher     gallaghe@teleport.com            }
{                                                                     }
{ The code in this unit uses TRegIniFile for reading and writing      }
{ information from the registry. Since I want the code to work with   }
{ various versions of Delphi, the constants hold key information      }
{ except for the part which indicates the version of Delphi installed }
{ The function [GetDelphiInfo] is used to try and determine which     }
{ version of Delphi is currently installed. If there are more then    }
{ one version, the first version is assumed.                          }
{                                                                     }
{ NOTE: The read and write routines/wrappers could be optimized. This }
{ job I have left to you. Why, so that you can learn from it. If you  }
{ get stuck then try again, if you are really stuck, send me a email  }
{ about the problem. Struggling is good for learning.                 }
{                                                                     }
{ NOTE: This entire unit is part of a project I created for managing  }
{ Delphi projects visually. It is located at                          }
{ http://www.delphifreestuff.com/                                     }
{                                                                     }
{ provided free of charge (not public domain)                         }
{ NOT TO BE SOLD OR DISTRIBUTED FOR A CHARGE without my permission.   }
{                                                                     }
{ No warranty is implied and we are not responsible for any problems  }
{ that might arise from the use of this unit. You use this unit       }
{ entirely at your own risk.                                          }
{                                                                     }
{ 03/12/2000                                                          }
{   Changed lots of stuff so that more versions of Delphi can be used }
{   without requiring hard coded constants.                           }
{ 03/27/2000                                                          }
{   Added GetDelphiDirectHeadLines procedure                          }
{ 06/06/2000                                                          }
{   Made compatible with D5, hope that doesn't break older versions.  }
{*********************************************************************}

{..$DEFINE KG_DEBUG}

interface

uses
  Windows, Messages, SysUtils, Classes,Controls, StdCtrls,FileCtrl,
  Registry {$IFDEF KG_DEBUG },Dialogs{$ENDIF};


type
  TDelphiInfo = Record
    ExeName:String ;
    Version:String ;
    InstallPath:String ;
  end ;

type
  TSessionObj = Class(TObject)
    constructor Create(Param1: String; Param2: String) ;
  public
    cParam1: String ;
    cParam2: String ;
  end ;

function GetDelphiInfo(var recInfo: TDelphiInfo):boolean ;

function GetLocale:String ;
function DelphiLastSession:String ;
function DelphiProjects:TStringList ;
function GetSearchPath:String ;
function GetProject(cName:String):String ;
function SetSession(S:String):Boolean ;
function FlushProject:Boolean;
function DeleteClosedProjects:Boolean;

function GetDelphiDirectHeadLines: String ;

function Len(S:String):LongInt;
function LastChrInStr(const S: String): Char;

procedure ReadCodeInSight;

function WhereIsDelphi:String;
function DelphiExe:String;
function DelphiVersionStr: String ;

procedure kg_FlushVars;

function SubStr(const S: string; C: Char):String;

procedure SetRegStringValue(_RootKey: HKEY;
                      _Localkey,
                      _Field: String;
                      V:String);

function GetRegStringValue(_RootKey: HKEY;
                     _Localkey,
                     _Field: String;
                     V:String):String;


const
  xUserKey     = HKEY_CURRENT_USER ;
  xMachince    = HKEY_LOCAL_MACHINE ;
  SessionKey   = 'Software\Borland\Delphi\%s\Session' ;
  SearchPath   = 'Software\Borland\Delphi\%s\Library' ;
  ProjectsKey  = 'Software\Borland\Delphi\%s\Closed Projects' ;
  xLocalDelphi = 'Software\Borland\Delphi\%s' ;
  xField       = 'Project' ;
  xDef         = 'ERROR' ;
  MAX_CODE     = 500 ;
  LF           = #10 ;
  CR           = #13 ;
  CRLF         : PChar = CR+LF ;
  SingleQuote  = #39 ;
var
  aList:TStringList ;
  aLines:TStringList ;
  aString:array[0..MAX_CODE,0..2] of String ;

implementation

constructor TSessionObj.Create(Param1: String; Param2: String) ;
begin
  Inherited Create;
  cParam1 := Param1;
  cParam2 := Param2;
end ;

function DelphiVersionStr: String ;
var
  R: TDelphiInfo ;
begin
  if GetDelphiInfo(R) then
    Result := R.Version 
  else
    raise Exception.Create('Failed to get Delphi version') ;
end ;

function GetDelphiInfo(var recInfo: TDelphiInfo):boolean ;
const
  LocalHome = 'Software\Borland\Delphi' ;
  FoundNoHome = 'Failed to find Delphi key in HKEY_LOCAL_MACHINE' ;
var
  Reg:      TRegistry ;
  aList:    TStringList ;
  cDelphi:  String ;
  iCount:   Integer ;
  i:        Integer ;
begin
  with RecInfo do
  begin
    ExeName := '?' ;
    Version := '?' ;
    InstallPath := '?' ;
  end ;

  Reg := TRegistry.Create ;
  try
    with Reg do
    begin
      RootKey := HKEY_LOCAL_MACHINE ;
      if OpenKey(LocalHome, False) then
      begin
        if HasSubKeys then
        begin
          aList := TStringList.Create ;
          try
            GetKeyNames(aList) ;
            iCount := aList.Count ;
            if iCount > -1 then
            begin
              { Attempt to get the first valid version of Delphi installed }
              for i := 0 to iCount -1 do
              begin
                CloseKey ;

                if OpenKey(LocalHome + '\' + aList.Strings[i], False) then
                begin
                  try
                    if ValueExists('App') then
                      cDelphi := ReadString('App')
                    else
                    {$IFDEF DELPHI_5}
                      cDelphi := ReadString('Delphi ' + aList.Strings[i][1]) ;
                    {$ELSE}
                    cDelphi := ReadString('Delphi ' + aList.Strings[i][1]) ;
                    {$ENDIF}
                    CloseKey ;
                    if FileExists(cDelphi) then
                    begin
                      if OpenKey(LocalHome + '\' + aList.Strings[i], False) then
                      begin
                        recInfo.InstallPath := ReadString('RootDir') ;
                        recInfo.ExeName := cDelphi ;
                        recInfo.Version := aList.Strings[i] ;
                        CloseKey ;
                        break ;
                      end ;
                    end ;
                  except
                    { Don't care to surface the exception to the UI }
                    CloseKey ;
                  end ;
                end ;
              end ;
            end ;
          finally
            aList.Free ;
          end ;
        end ;
      end else
        raise Exception.Create(FoundNoHome) ;
    end ;
  finally
    Reg.CloseKey ;
  end ;

  Result := True ;
end;

function GetDelphiVersion(FileName:String):String ;
var
  BufSize     : Integer ;
  InfoBuf     : PChar ;
  VerBufSize  : Integer ;
  VerBuf      : PChar ;
begin
  Result := '0.0' ;

  BufSize := GetFileVersionInfoSize(PChar(FileName),DWORD(BufSize)) ;
  if BufSize > 0 then
  begin
    InfoBuf := AllocMem(BufSize) ;
    try
      GetFileVersionInfo(PChar(FileName),0,BufSize,InfoBuf) ;

      { NOTE the use of function GetLocale (within this unit) }
      VerQueryValue(InfoBuf,
        PChar('StringFileInfo\' + GetLocale + '04E4\ProductVersion'),
        Pointer(VerBuf),DWORD(VerBufSize)) ;
        
      Result := VerBuf ;
    finally
      FreeMem(InfoBuf, BufSize) ;
    end ;
  end ;

end ;

procedure ReadCodeInSight;
  function RemCrLf(const S: String): String;
  var i:Integer;
  begin
    Result:=S;
    { Some strings of dual CRLF's }
    for i := 1 to 4 do
      if (LastChrInStr(Result)=LF) or (LastChrInStr(Result)=CR) then
        Delete(Result,Length(Result) -1,2);
  end;
var
  F:Textfile;
  cDir:String;
  S,cLine:String;
  cFile:String;
  i,x:Integer;
begin
  cFile :=  'Delphi32.dci';
  S := '';
  cDir := WhereIsDelphi + #92  + 'Bin';
  i := 0;

  if not DirectoryExists(cDir) then
    raise Exception('Could not locate ' + cDir) ;

  if not FileExists(cDir + #92 + cFile) then
    raise Exception('Could not locate ' + cFile) ;

  try
    AssignFile(F,cDir + #92 + cFile);
    Reset(F);

    ReadLn(F,cLine);

    repeat
      if Copy(cLine,1,1) = #91 then
      begin
        x := Pos('|',cLine) ;
        aString[i,0] := Copy(cLine,2,x -2 ) ;
        aString[i,1] := Copy(cLine,x + 2,Len(cLine)- (x +2)) ;
        ReadLn(F,cLine) ;
      end else
      begin
        repeat
          aString[i,2] := aString[i,2] + cLine + CRLF ;
          ReadLn(F,cLine) ;
          if eof(F) then
          begin
            aString[i,2] := aString[i,2] + cLine + CRLF ;
            break ;
          end ;
        until (Copy(cLine,1,1) = #91) ; // [
        aString[i,2] := RemCrLf(aString[i,2]) ;
        Inc(i) ;
      end;
    until (eof(F)) ;
    if len(cLine) > 0 then
      aString[i,2] := aString[i,2] + cLine ;
    aString[i,2] := RemCrLf(aString[i,2]) ;
  finally
    Closefile(F) ;
  end ;
end ;

function WhereIsDelphi:String;
const
  NoHome = 'Error getting Delphi home';
var
  recInfo: TDelphiInfo ;
  cKey: String ;
begin
  if GetDelphiInfo(recInfo) then
  begin
    cKey := Format(xLocalDelphi,[RecInfo.Version]) ;
    Result := GetRegStringValue(xMachince,cKey,'RootDir',NoHome) ;
  end ;
end;

{ KSG 03.12.2000 - Rewrote entire function }
function DelphiExe:String;
const
  xExeErr = 'Failed to get executable name';
var
  Reg: TRegistry ;
  cTmp:String ;
begin
  Result := xExeErr ;
  Reg := TRegistry.Create ;
  try
    with Reg do
    begin
      RootKey := HKEY_LOCAL_MACHINE ;
      if OpenKeyReadOnly(Format(xLocalDelphi,[DelphiVersionStr])) then
      begin
        cTmp := 'Delphi ' + DelphiVersionStr[1] ;
        Result := ReadString(cTmp) ;
        CloseKey ;
      end ;
    end ;
  finally
    Reg.Free ;
  end ;
end;

{------------------------------------------------------------------------------
     Function:  DelphiProjects
  Description:  Retrieves Known Delphi "Closed Projects" from the registry
       Author:  Kevin S. Gallagher - gallaghe@teleport.com
 Date created:  03/31/2000
Date modified:
      Purpose:
 Known Issues:  None
     Comments:  The caller must free up the attached objects when done.
 ------------------------------------------------------------------------------}
function DelphiProjects:TStringList;
var
  S:String;
  Tmp:String;
  cName:String;
  cParam1,
  cParam2: String ;
  x:Integer;
  iPos:Integer;
begin
  aList.Clear;
  for x := 0 to 4 do
  begin
    S:= GetProject('File_' + IntToStr(x)) ;

    iPos := Pos(xDef,S) ;
    if iPos = 0 then
    begin
      Tmp := Copy(S,Pos(SingleQuote,S)+1 ,Len(S) -  Pos(SingleQuote,S));
      cName := Copy(Tmp,1,Pos(SingleQuote,Tmp) -1) ;
      { This is the type of module i.e. "SourceModule" }
      cParam1 := Copy(S,1,Pos(',',S) -1) ;
      iPos := Pos(',',Tmp) ;
      { Some unknown parameters tacked on after the project name }
      cParam2 := Copy(Tmp, iPos +1, Length(Tmp) - iPos) ;
      aList.AddObject(cName,TSessionObj.Create(cParam1,cParam2)) ;
    end;
  end;

  { You might wanted to check this yourself? }
  if aList.Count = 0 then
    raise Exception.Create('Failed to retreive any projects');

  Result := aList;
end;

function GetSearchPath:String;
var
  cKey    : String ;
  recInfo : TDelphiInfo ;
  Reg     : TRegistry ;
  cSearch : String ;
begin
  if GetDelphiInfo(recInfo) then
  begin
    cKey := Format(SearchPath,[RecInfo.Version]) ;
    Reg  := TRegistry.Create ;
    try
      with Reg do
      begin
        RootKey := xUserKey ;
        if OpenKey(cKey, False) then
        begin
          {$IFDEF DELPHI_5}
          cSearch := 'Search Path' ;
          {$ELSE}
          cSearch := 'SearchPath' ;
          {$ENDIF}
          if ValueExists(cSearch) then
          begin
            {$IFDEF DELPHI_5}
            Result := StringReplace(ReadString(cSearch),
                        '$(DELPHI)',
                        WhereIsDelphi,
                        [rfReplaceAll,rfIgnoreCase]) ;
            {$ELSE}
            Result := ReadString(cSearch) ;
            {$ENDIF}
            CloseKey ;
          end ;
        end ;
      end ;
    finally
      Reg.Free ;
    end ;
  end ;
end;

function GetProject(cName:String):String;
var
  recInfo: TDelphiInfo ;
  Reg: TRegistry ;
  cKey: String ;
begin
  if GetDelphiInfo(recInfo) then
  begin
    cKey := Format(ProjectsKey,[RecInfo.Version]) ;
    Reg := TRegistry.Create ;
    try
      if Reg.OpenKeyReadOnly(cKey) then
      begin
        Result := Reg.ReadString(cName) ;
        Reg.CloseKey ;
      end else raise Exception.Create('Failed to open HKCU\' + cKey) ;
    finally
      Reg.Free ;
    end ;
  end else raise Exception.Create('Failed to get Delphi version') ;
end;

function SetSession(S:String):boolean;
var
  cComp:String;
  cKey: String ;
  recInfo: TDelphiInfo ;
begin
  Result := False ;

  if GetDelphiInfo(recInfo) then
  begin
    cKey := Format(SessionKey,[RecInfo.Version]) ;
    SetRegStringValue(xUserKey,cKey,xField,S);
    cComp := GetRegStringValue(xUserKey,cKey,xField,'');
    if cComp = S then Result := True;
  end ;

end;

function DelphiLastSession:String;
var
  recInfo: TDelphiInfo ;
  Reg: TRegistry ;
begin
  if GetDelphiInfo(recInfo) then
  begin
    Reg := TRegistry.Create ;
    try
      if Reg.OpenKeyReadOnly(Format(SessionKey,[RecInfo.Version])) then
      begin
        Result := Reg.ReadString(xField);
        Reg.CloseKey ;
      end ;
    finally
      Reg.Free ;
    end ;
  end else raise Exception.Create('Failed to get Delphi Last session') ;

end;

function FlushProject:boolean;
var
  recInfo: TDelphiInfo ;
begin
  if GetDelphiInfo(recInfo) then
    SetRegStringValue(xUserKey,Format(SessionKey,[RecInfo.Version]),xField,'') ;

  Result := (Len(DelphiLastSession) = 0) ;
end;

function DeleteClosedProjects:Boolean ;
var
  Reg : TRegistry ;
  i : Integer ;
begin
  Result := False ;
  Reg := TRegistry.Create ;
  try
    if Reg.OpenKey(Format(ProjectsKey,[DelphiVersionStr]),False) then
    begin
      for i := 0 to 4 do
        if Reg.ValueExists('File_' + IntToStr(i)) then
          if not Reg.DeleteValue('File_' + IntToStr(i)) then
          begin
            Reg.CloseKey ;
            raise Exception.Create('Failed to remove File_'+ IntToStr(i)) ;
          end ;
      Reg.CloseKey ;
    end ;
    Result := True ;
  finally
    Reg.Free ;
  end ;
end ;

procedure SetRegStringValue(_RootKey: HKEY;
                      _Localkey, _Field: String; V: String);
var
  Reg: TRegistry ;
begin
  Reg := TRegistry.Create ;
  try
    Reg.RootKey := _RootKey;
    if Reg.OpenKey(_Localkey,true) then
    begin
      Reg.WriteString(_Field,V);
      Reg.CloseKey;
    end ;
  finally
    Reg.Free;
  end ;
end;

function GetRegStringValue(_RootKey: HKEY;
                     _Localkey, _Field: String; V:String):String;
var
  {$IFDEF DELPHI_5}
  Reg : TRegistry ;
  {$ELSE}
  TR: TRegIniFile;
  {$ENDIF}
begin
  Result := V;

  {$IFDEF DELPHI_5}
  Reg := TRegistry.Create ;
  try
    with Reg do
    begin
      RootKey := _RootKey ;
      if OpenKey(_LocalKey,False) then
      begin
        if ValueExists(_Field) then
          Result := ReadString(_Field)
      end ;
    end ;
  finally
    Reg.Free ;
  end ;
  {$ELSE}
  TR := TRegIniFile.Create('');
  with TRegistry(TR) do begin
    RootKey := _RootKey;
    if OpenKey(_Localkey,false) then begin
      if ValueExists(_Field) then
        Result := ReadString(_Field);
      CloseKey;
     end;
   end;
  TR.Free;
  {$ENDIF}
end;

{------------------------------------------------------------------------------
     Function:  GetDelphiDirectHeadLines
  Description:  Gets raw string which is used in Delphi Direct IDE add-in
       Author:  Kevin S. Gallagher - gallaghe@teleport.com
 Date created:  03/27/2000
Date modified:
      Purpose:  Once formatted the string can be shown in a Treeview
 Known Issues:  None
     Comments:  Requires massaging once the string is retrieved.
                The user most have connected to Borland at least once for the
                code to work.
 ------------------------------------------------------------------------------}
function GetDelphiDirectHeadLines: String ;
const
  TheKey = 'Software\Borland\Delphi\%s\Direct' ;
var
  Reg: TRegistry ;
begin
  Result := '' ;
  Reg := TRegistry.Create ;
  try
    if Reg.OpenKey(Format(TheKey,[DelphiVersionStr]),False) then
    begin
      if Reg.ValueExists('HeadLines') then
      begin
        Result := Reg.ReadString('HeadLines') ;
        Reg.CloseKey ;
      end ;
    end ;
  finally
    Reg.Free ;
  end ;
end ;

{------------------------------------------------------------------------------
     Function:  GetLocale
  Description:  Used to obtain the country code for the current user
       Author:  Kevin S. Gallagher - gallaghe@teleport.com
 Date created:  02/20/2000
Date modified:
      Purpose:
 Known Issues:  None
     Comments:  None
 ------------------------------------------------------------------------------}
function GetLocale:String ;
var
  Reg: TRegistry ;
begin
  Reg := TRegistry.Create ;
  try
    with Reg do
    begin
      RootKey := HKEY_CURRENT_USER ;
      OpenKey('Control Panel\Desktop\ResourceLocale', False) ;
      Result := ReadString('') ;
      Result := Copy(Result,5,4) ;
      CloseKey ;
    end ;
  finally
    Reg.free ;
  end ;
end ;

function Len(S:String):LongInt;
begin
  Result := Length(S);
end;

function  LastChrInStr(const S: String): Char;
begin
  Result := S[Length(S)];
end;

function DelChars(const S: string; Chr: Char): string;
var
  I: Integer;
begin
  Result := S;
  for I := Len(Result) downto 1 do begin
    if Result[I] = Chr then Delete(Result, I, 1);
  end;
end;

function SubStr(const S: string; C: Char):String;
begin
  Result :=  Copy(S,Pos(C,S) +1 ,Len(S) -  Pos(C,S));
end;

procedure kg_FlushVars ;
begin
  {aList.Free ;
  aLines.Free ;}
end;

initialization
  aList := TStringList.Create ;
  aList.Duplicates := dupIgnore ;
  aLines:= TStringList.Create ;

finalization
  aList.Free;
  aLines.Free;

end.
