unit KG_VariousDialogs;

{*****************************************************************************}
{ Author:                                                                     }
{   Kevin S. Gallagher									                                      }
{ Description:                                                                }
{   These functions are alternate method which can be used to display 	      }
{   messages rather then using MessageDlg or ShowMessage. The major           }
{   advantage with these functions is that you can:				                    }
{     1) Center messages on the form which invoked them.				              }
{		  2) Takes advantage of Delphi 4 Overloading and default parameters.	    }
{														                                                  }
{ All functions have been tested with D3 and D4							                  }
{                                                                             }
{ Comments:                                                                   }
{   06/11/2000                                                                }
{   There are defines for VER130, this is for D5, change them to your version }
{   if not using D5, see JCL.INC for versioning.                              }
{*****************************************************************************}

interface

uses  Windows, Messages, SysUtils, Controls, Forms, Dialogs, StdCtrls;


function DefMessageDlg(const aCaption: string; const Msg: string;
                       DlgType: TMsgDlgType;
                       Buttons: TMsgDlgButtons;
                       DefButton: Integer;
                       HelpCtx: Longint): Integer;


{$IFDEF VER130}
function MessageDlgCtr(const Msg: string; DlgType: TMsgDlgType;
                       Buttons: TMsgDlgButtons;
                       bCenterOnForm: boolean = True): Integer; overload ;

function MessageDlgCtr(const aCaption: string; const Msg: string;
                       DlgType: TMsgDlgType;
                       Buttons: TMsgDlgButtons;
                       HelpCtx: Longint;
                       bCenterOnForm: boolean = True): Integer; overload ;


{$ELSE}
function MessageDlgCtr(const aCaption: string; const Msg: string;
                       DlgType: TMsgDlgType;
                       Buttons: TMsgDlgButtons;
                       HelpCtx: Longint;
                       bCenterOnForm: boolean): Integer ;

{$ENDIF}

implementation

function kgStrCrLf(const cMsg:String):String;
var nLen,i:Integer;
begin

  nLen := Length(cMsg);
  i := 1;
  while i <= nLen do begin
    if cMsg[i] in [';','~'] then
      Result := Result + #13
    else
      Result := Result + cMsg[i];
    Inc(i);
  end;
end;


function DefMessageDlg(const aCaption: string; const Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons; DefButton: Integer; HelpCtx: Longint): Integer;
var
  i: Integer;
  btn: TButton;
begin
  with CreateMessageDialog(kgStrCrLf(Msg), DlgType, Buttons) do
  try
    Caption := aCaption ;
    HelpContext := HelpCtx ;
    for i := 0 to ComponentCount -1 do begin
      if Components[i] Is TButton then begin
        btn := TButton(Components[i]) ;
        btn.default:= btn.ModalResult = DefButton ;
        if btn.default then
          ActiveControl := Btn ;
      end;
    end ;
    Result := ShowModal ;
  finally
    Free ;
  end ;
end;


{$IFDEF VER130}
function MessageDlgCtr(const Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons;
                       bCenterOnForm: boolean = True): Integer; overload ;
begin
  if Screen.ActiveForm = nil then bCenterOnForm := false ;

  with CreateMessageDialog(kgStrCrLf(Msg), DlgType, Buttons) do
    try
      if bCenterOnForm then begin
        Left := Screen.ActiveForm.Left + (Screen.ActiveForm.Width div 2) -(Width div 2) ;
        Top := Screen.ActiveForm.Top + (Screen.ActiveForm.Height div 2) -(Height div 2) ;
      end ;
      Result := ShowModal ;
    finally
      Free ;
    end;
end;

function MessageDlgCtr(const aCaption: string; const Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons;
  HelpCtx: Longint; bCenterOnForm: boolean = True): Integer;
begin
  with CreateMessageDialog(kgStrCrLf(Msg), DlgType, Buttons) do
    try
      Caption := aCaption ;
      HelpContext := HelpCtx;

      if Screen.ActiveForm = nil then bCenterOnForm := false ;
      if bCenterOnForm then begin
        try
          Left := Screen.ActiveForm.Left + (Screen.ActiveForm.Width div 2) -(Width div 2) ;
          Top := Screen.ActiveForm.Top + (Screen.ActiveForm.Height div 2) -(Height div 2) ;
        except
        end ;
      end ;
      Result := ShowModal ;
    finally
      Free ;
    end ;
end;
{$ELSE}
function MessageDlgCtr(const aCaption: string; const Msg: string;
                       DlgType: TMsgDlgType;
                       Buttons: TMsgDlgButtons;
                       HelpCtx: Longint;
                       bCenterOnForm: boolean): Integer ;
begin
  with CreateMessageDialog(kgStrCrLf(Msg), DlgType, Buttons) do
    try
      Caption := aCaption ;
      HelpContext := HelpCtx;
      if Screen.ActiveForm = nil then bCenterOnForm := false ;
      if bCenterOnForm then begin
        try
          Left := Screen.ActiveForm.Left + (Screen.ActiveForm.Width div 2) -(Width div 2) ;
          Top := Screen.ActiveForm.Top + (Screen.ActiveForm.Height div 2) -(Height div 2) ;
        except
        end ;
      end ;
      Result := ShowModal ;
    finally
      Free ;
    end ;
end;
{$ENDIF}
end.
