
{*********************************************************************}
{ Description:  Project Manager                                       }
{  Programmer:  Kevin S Gallagher     gallaghe@teleport.com           }
{    Requires:  TBrkApart component (FreeWare)                        }
{               MkTable              "     "                          }
{               ItemProp             "     "                          }
{    Versions:  3 and 4 of Delphi and higher                          }
{---------------------------------------------------------------------}
{ Revisions/Fixs                                                      }
{ KSG 12/09/1997    DPRegstr.pas  (see file header)                   }
{ KSG 12/09/1997    frmAddTo.pas  (see file header)                   }
{ KSG 12/20/1997    Added code for Explorer drag-n-dropping projects  }
{ KSG 03/18/1998    Added new method to view units and DFM files      }
{                   also added Brad Stower's ItemProp unit.           }
{ KSG 03/21/1999    Updated to work with D4                           }
{ KSG 03/18/2000    Added code to position on last project focused    }
{                   when the application closed.                      }
{                   Replaced the ShellAbout with my own simple about. }
{ KSG 04/02/2000    See Changed.txt                                   }
{*********************************************************************}
unit Main;

{$DEFINE KG_TESTING}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DB, DBTables, StdCtrls, Grids, DBGrids, DBCtrls, Mask, Menus,
  ComCtrls,FileCtrl, Buttons, ExtCtrls, MKTable,ShellAPI, ToolWin, MenuBar; //, GradForm;

type
  TfrmMain = class(TForm)   // TGradientForm
    DataSource1: TDataSource;
    grdProjects: TDBGrid;
    DBMemo1: TDBMemo;
    dbEditPath: TDBEdit;
    mnuMainMenu1: TMainMenu;
    mnuMain: TMenuItem;
    mnuRunProject: TMenuItem;
    mnuExit: TMenuItem;
    mnuDelete: TMenuItem;
    mnuAddproject: TMenuItem;
    mnuEditproject: TMenuItem;
    mnuAppHelp: TMenuItem;
    ToolBarPnl1: TPanel;
    cbChangeIndex: TComboBox;
    cbLangFilter: TComboBox;
    StatusBar1: TStatusBar;
    cmdProjectView: TButton;
    Table1: TTableDBF;
    mnuPackTable: TMenuItem;
    cmdEditProject: TButton;
    cmdNewProject: TButton;
    cmdRcentFileList: TButton;
    cmdExit: TButton;
    mnuGridPop: TPopupMenu;
    mnuPopViewUnits: TMenuItem;
    mnuPopEditProject: TMenuItem;
    mnuReindexTable: TMenuItem;
    mnuPopUpDelete: TMenuItem;
    mnuIndex: TMenuItem;
    mnuAbout: TMenuItem;
    N1: TMenuItem;
    mnuFind: TMenuItem;
    N2: TMenuItem;
    mnuMisc: TMenuItem;
    ViewknownDPKfile1: TMenuItem;
    MenuBar1: TMenuBar;
    mnuCodeInSight: TMenuItem;
    mnuDelphiDirect: TMenuItem;
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure mnuDeleteClick(Sender: TObject);
    procedure mnuExitClick(Sender: TObject);
    procedure mnuRunProjectClick(Sender: TObject);
    procedure mnuAddprojectClick(Sender: TObject);
    procedure mnuEditprojectClick(Sender: TObject);
    procedure cmdEditProjectClick(Sender: TObject);
    procedure grdProjectsDblClick(Sender: TObject);
    procedure cmdNewProjectClick(Sender: TObject);
    procedure cmdRcentFileListClick(Sender: TObject);
    procedure cbChangeIndexChange(Sender: TObject);
    procedure ToggleCmdStatus(lFlag: Boolean);
    procedure grdProjectsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
    procedure grdProjectsDrawDataCell(Sender: TObject; const Rect: TRect;
      Field: TField; State: TGridDrawState);
    procedure cbLangFilterChange(Sender: TObject);
    procedure cmdProjectViewClick(Sender: TObject);
    procedure mnuPackTableClick(Sender: TObject);
    procedure mnuReindexTableClick(Sender: TObject);
    procedure mnuIndexClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure mnuFindClick(Sender: TObject);
    procedure ViewknownDPKfile1Click(Sender: TObject);
    procedure mnuCodeInSightClick(Sender: TObject);
    procedure mnuDelphiDirectClick(Sender: TObject);
  private
    bFirstTime: boolean ;
    {$IFDEF DFS_DELPHI_3}
    MaxPt: TPoint;
    iSignal: Integer;
    procedure WmGetMinMaxInfo(var Msg: TWmGetMinMaxInfo);
      message WM_GETMINMAXINFO;
    {$ENDIF}
    function  WhereWinDir:String;
    function OpenProjectTable(lMode: Boolean): Boolean;
    function ManagerIniCheck:boolean;
    procedure DrawField(const Value: String; const Rect: TRect;
      vCanvas: TCanvas; vFont: TFont; vAlignment: TAlignment);
    procedure grdProjectsDrawColumnCell(Sender: TObject;
      const Rect: TRect; DataCol: Integer; Column: TColumn;
      State: TGridDrawState);
    procedure AddDroppedProject(const Info: String);
    procedure wmDropFiles(var Message : TWMDropFiles);
      message WM_DROPFILES;
  public
    { Public declarations }
    procedure AppOnHint(Sender: TObject);
    function ProjectInQuestion:String;
  end;

const
  HELP_TAB = 15 ;
var
  frmMain  : TfrmMain;
  sPath    : String;
  cVBasic16: String;
  cDelphi16: String;

implementation

uses frmAddTo,
     IniFiles,
     Global,
     kg_VariousDialogs,
     ReadDpr,
     TableCode,
     DelphiInfo,
     TableCreate,
     Registry,
     DPRegstr,
     ReadPackage,
     MyAbout,
     ProjectsFromPackFiles,
     ReopenProjectList,
     InSightCode,
     Unit9;

{$R *.DFM}

procedure CenterForm(f : TForm) ;
begin
  f.Left := Screen.ActiveForm.Left + (Screen.ActiveForm.Width div 2) - (f.Width div 2) ;
  f.Top := Screen.ActiveForm.Top + (Screen.ActiveForm.Height div 2) - (f.Height div 2) ;
end ;

procedure TfrmMain.mnuRunProjectClick(Sender: TObject);
var
  cDir : String;
  cMake: String;
  cLang: String;
  cRun : String;
  cType: String;
  pRun : PChar;
begin

  {$IFDEF KG_TESTING}
  if IsDelphiRunning then
    raise Exception.Create('Delphi is currently running!!');
  {$ENDIF}

  if Len(LanguageType) = 0 then
  begin
    WarnMsg('Missing language type');
    exit;
  end else
    cLang:= Trim(LanguageType);

  if Len(MakeFileName) = 0 then
  begin
    WarnMsg('Missing MakeFile name');
    exit;
  end else
    cMake:= Trim(MakeFileName);

  cDir  := WhereIsProject;
  if not DirectoryExists(cDir) then
  begin
    WarnMsg('Cannot find project;' + cDir);
    Exit;
  end;

  if not FileExists(cDir + '\' + cMake) then
  begin
    WarnMsg('Cannot find projects makefile;' + cDir + '\' + cMake);
    Exit;
  end;

  {$IFDEF KG_TESTING}
    frmMain.Visible := False;
  {$ENDIF}

  cType := Table1.FieldByName('WinType').AsString;

  if UpperCase(cLang) = 'DELPHI' then
  begin
    if cType = '16' then
    begin
      if not DirectoryExists(cDelphi16) then
      begin
        Exit;
      end;
      cRun  := cDelphi16 + D16 + ' ' + cMake;
      pRun  := StrAlloc(300);
      try
        StrPCopy(pRun,cRun);
        chdir(cDir);
        WinExec(pRun,SW_ShowNormal);
      finally
        StrDispose(pRun);
      end;
      Close;
    end;
    if cType = '32' then
    begin
      cRun := cMake;
      RunWinApp(cDir + '\' + cRun );
      Close;
    end;
  end else if UpperCase(cLang) = 'VB' then
  begin
    if IsProject16Bit then
    begin
      if not DirectoryExists(cVBasic16) then
      begin
        frmMain.Visible := True;
        Exit;
      end;
      cRun  := cVBasic16 + '\vb.exe ' + #34 + cDir + '\' + cMake + #34;
      pRun  := StrAlloc(300);
      try
        StrPCopy(pRun,cRun);
        chdir(cDir);
        WinExec(pRun,SW_ShowNormal);
      finally
        StrDispose(pRun);
      end;
      Close;
    end;

    if cType = '32' then
    begin
      cRun := cMake;
      RunWinApp(cDir + '\' + cRun );
      Close;
    end;
  end else
  begin
    WarnMsg('This language is not supported');
    cLang := 'UNKNOWN';
  end;
end;

procedure TfrmMain.mnuAddprojectClick(Sender: TObject);
var
  f:TfrmEdit;
begin

  frmMain.Tag := ADD_NEW_PROJECTS;
  Table1.refresh;

  f := TfrmEdit.Create(Application);
  try
    f.ShowModal;
    grdProjects.Setfocus;
  finally
    f.Free;
  end;
end;

procedure TfrmMain.mnuEditprojectClick(Sender: TObject);
var
  f:TfrmEdit;
begin
  frmMain.Tag := EDIT_CURRENT_PROJECT;
  Table1.refresh;

  f := TfrmEdit.Create(Application);
  try
    f.ShowModal;
  finally
    grdProjects.Setfocus;
    f.Free;
  end;
end;

{ Make sure not to have Table1 open in the IDE }
procedure TfrmMain.FormActivate(Sender: TObject);
var
 cTmp : String ;
 cKey : String ;
 Reg  : TRegistry ;
begin
  if not bFirstTime then exit ;
  grdProjects.Visible := False ;
  bFirstTime := False ;
  ToggleCmdStatus(False);

  { Create table and indexs if no table in apps directory }
  if not FileExists(sPath + '\' + MainDbf) then
  begin
    try
      CreateMainDBF;
      if Table1.Active then
        Table1.Close;
    except on E: EDatabaseError do begin
      ErrMsg('ERROR creating table;' + E.Message);
      exit;
      end;
    end;
  end;
  
  Self.Refresh;

  cTmp := AppPath + '\' + ChangeFileExt(MainDbf, '.PX');

  { Insures the main index exist }
  if not FileExists(cTmp) then
  begin
    if Table1.Active then
      Table1.Close;
    if not OpenProjectTable(False) then
    begin
      ErrMsg('Could not create' + #13 + cTmp);
      exit;
    end;
    Table1.Close;
  end;
  cTmp := '';

  { This is were the table is normally opened }
  if not OpenProjectTable(False) then
  begin
    ErrMsg('Failed opening main table for sharing');
    exit;
  end else
  begin
    with cbChangeIndex do
    begin
      Items.Add('Project/Language');
      Items.Add('Name/Language');
      Items.Add('Order by Language');
      Items.Add('Natural order');
    end ;

    with cbLangFilter do
    begin
      Items.Add('No filter');
      Items.Add('Delphi');
      Items.Add('Visual Basic');
    end ;

    ToggleCmdStatus(True);
    Table1.IndexName := 'ByNameLanguage';

    { Attempt to position ourselfs on the last project which
      had focus when the application termintated. See companion
      code in FormClose event. }
    cKey := '' ;
    if Table1.Active then
    begin
      Reg := TRegistry.Create ;
      try
        if Reg.OpenKeyReadOnly('Gallagher') then
          if Reg.OpenKeyReadOnly('Project Manager') then
          begin
            cKey := Trim(Reg.ReadString('Last Viewed Project')) ;
            Reg.CloseKey ;
            if Length(cKey) >0 then
              Table1.Locate('Name',cKey, [loPartialKey]);
          end ;
      finally
        Reg.Free ;
      end ;
    end ;

    SetScrollPos(grdProjects.Handle,SB_VERT,0,True);
  end;

  if Table1.RecordCount > 0 then
  begin
    cmdEditProject.Enabled  := True;
    mnuDelete.Enabled       := True;
    mnuRunProject.Enabled   := True;
  end else
  begin
    cmdEditProject.Enabled  := False;
    mnuDelete.Enabled       := False;
    mnuRunProject.Enabled   := False;
  end;

  if not ManagerIniCheck then
    exit;

  // grdProjects.Columns.Items[0].Color := ???;
  // grdProjects.FixedColor := clAqua;

  cbChangeIndex.Style    := csDropDownList;
  cbLangFilter.Style     := csDropDownList;
  cbChangeIndex.ItemIndex:= 1;
  cbLangFilter.ItemIndex := 0;

  grdProjects.Options    := [dgEditing,dgRowSelect,dgColLines,
                             dgConfirmDelete,dgCancelOnExit];

  grdProjects.DataSource := DataSource1;
  grdProjects.Visible := True ;
  frmMain.Caption  := 'Project Manager';
  ActiveControl := grdProjects ;
end;

procedure TfrmMain.cmdEditProjectClick(Sender: TObject);
begin
  try
    frmMain.mnuEditproject.Click;
  except
    ErrMsg('Failed to edit project!!!');
  end;
end;

procedure TfrmMain.grdProjectsDblClick(Sender: TObject);
begin
  frmMain.mnuRunProjectClick(Self);
end;

procedure TfrmMain.cmdNewProjectClick(Sender: TObject);
begin
  frmMain.mnuAddproject.Click;
end;

procedure TfrmMain.cmdRcentFileListClick(Sender: TObject);
var
  f:TfrmProjectListing ;
begin
  f := TfrmProjectListing.Create(Application);
  try
    CenterForm(f) ;
    f.ShowModal;
  finally
    f.Free;
    grdProjects.SetFocus;
  end;
end;

procedure TfrmMain.cbChangeIndexChange(Sender: TObject);
var
  S:String;
begin

  case cbChangeIndex.ItemIndex of
    0: S:= 'ByProjectLanguage';
    1: S:= 'ByNameLanguage';
    2: S:= 'Language';
    3: S:= '';
  end;

  try
    Table1.IndexName := S;
  except
    ErrMsg('Failed setting index order');
  end;

  // SetScrollPos(grdProjects.Handle,SB_VERT,0,True);
  grdProjects.UpDate;
  grdProjects.Setfocus;
end;

procedure TfrmMain.grdProjectsKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = vk_Return then
    mnuRunProjectClick(Self);
end;

function TfrmMain.ManagerIniCheck:boolean;
const
  ER = 'ERROR';
  KSG = 'Kevin S. Gallagher';
  Ini_Name = 'manager.ini';
var
  ConfigIni: TiniFile;
  cWinDir  : String;
  cKevin   : String;
begin

  cWinDir := WhereWinDir;
  ConfigIni := TIniFile.Create(Ini_Name);
  with ConfigIni do
  begin
    { If you'd like move information to the registry, or if I get
      enough request then I will implement the change. }
    if not FileExists(cWinDir + '\' + Ini_Name) then
    begin
      WriteString('WhereIs','VB16','C:\Program Files\Vb16');
      WriteString('WhereIs','Delphi16','C:\Program Files\Delphi16\Bin');
      WriteString('Information','Version','1.03');
      WriteString('Information','Author',KSG);
    end;
    
    cVBasic16 := ReadString('WhereIs','VB16',ER);
    cDelphi16 := ReadString('WhereIs','Delphi16',ER);
    cKevin    := ReadString('Information','Author',ER);

    if not (cKevin = KSG) or (cKevin = ER) then
    begin
      ErrMsg('Missing entry in ' + Ini_Name);
      ToggleCmdStatus(False);
    end;

    if (cVBasic16 = ER) or  (cDelphi16 = ER) then
    begin
      ErrMsg('Failed to read information from ' + Ini_Name + #13 +
                 '16bit languages may not work!');
    end;
  end;

  ConfigIni.Free;

  Result := True;
end;

function TfrmMain.WhereWinDir: String;
var
  cHome: String;
  b    : array[0..144] of char ;
begin
  if GetWindowsDirectory(b,sizeof(b)) >0 then
    cHome := StrPas(b)
  else
    raise Exception.Create('Failed to get Windows home directory') ;

  Result := cHome;
end;

procedure TfrmMain.DataSource1DataChange(Sender: TObject; Field: TField);
begin
  if not Table1.Active then exit;

  { This differs from all other hints which go into the status panel }
  dbEditPath.Hint := MakeFileName;
end;

procedure TfrmMain.cbLangFilterChange(Sender: TObject);
begin
  Table1.IndexName := 'Language';
  case cbLangFilter.ItemIndex of
    0:  Table1.CancelRange;
    1:  Table1.SetRange(['Delphi'],['Delphi']);
    2:  Table1.SetRange(['VB'],['VB']);
  end;
  cbChangeIndex.ItemIndex := 1;
  grdProjects.Setfocus;

end;

procedure TfrmMain.cmdProjectViewClick(Sender: TObject);
var
  f:TfrmMakeRead ;
  fp: TfrmProjectsInPackages ;
  aPackFile: TStringList ;
  aProjects: TStringList ;
  cPackFile: String ;
  cProject : String ;
  cItem    : String ;
  i, iCount: Integer ;
  iPos     : Integer ;
begin
  if not Table1.Active then exit;

  if UpperCase(LanguageType) = 'DELPHI' then
    { Guess I need to code for Delphi group files next }
    if UpperCase(ExtractFileExt(MakeFileName)) = '.DPR' then
    begin
      DprToRead(WhereIsProject + '\' + MakeFileName);
      f := TfrmMakeRead.Create(Self);
      try
        f.Top := Self.Top;
        f.Left := Self.Left;
        f.ShowModal;
      finally
        f.Free;
      end ;
      { I assume it is a Group, might not be...
        Need to work with this some before introducing it. I want the
        interface to look like the IDE UI. }
    end else if UpperCase(ExtractFileExt(MakeFileName)) = '.BPG' then
    begin
      cPackFile := WhereIsProject + '\' + MakeFileName ;
      if FileExists(cPackFile) then
      begin
        aPackFile := TStringList.Create ;
        aProjects := TStringList.Create ;
        try
          aPackFile.LoadFromFile(WhereIsProject + '\' + MakeFileName) ;
          iCount := aPackFile.Count ;
          for i := 0 to iCount -1 do
          begin
            cItem := Trim(aPackFile.Strings[i]) ;
            if cItem = '$(DCC)' then
            begin
              cItem := Trim(aPackFile.Strings[i -1]) ;
              iPos := Pos(':',cItem) ;
              if iPos >0 then
              begin
                cItem := Trim(Copy(cItem,iPos +1,Length(cItem) - iPos)) ; ;
                if (Pos(':',cItem) = 0) and (Pos('\',cItem) = 0) then
                begin
                  if FileExists(WhereIsProject + '\' + cItem) then
                    if UpperCase(ExtractFileExt(WhereIsProject + '\' + cItem)) = '.DPR' then
                      aProjects.Add(WhereIsProject + '\' + cItem) ;
                end else
                begin
                  cProject := ExpandFileName(cItem) ;
                  if FileExists(cProject) then
                    if UpperCase(ExtractFileExt(cProject)) = '.DPR' then
                    aProjects.Add(cProject) ;
                end ;
              end ;
            end ;
          end ;

          iCount := aProjects.Count ;
          if iCount <> -1 then
          begin
            fp := TfrmProjectsInPackages.Create(Self) ;
            try
              CenterForm(fp) ;
              fp.ListBox1.Items.Assign(aProjects) ;
              fp.ListBox1.ItemIndex := 0 ;
              fp.Caption := 'From ' + cPackFile ;
              fp.ShowModal ;
            finally
              fp.Free ;
            end ;
          end ;
        finally
          aPackFile.Free ;
          aProjects.Free ;
        end ;
      end ;
    end else Information('Currently not programmed from this project groups') ;

  grdProjects.Setfocus;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
var
  S:String ;
{$IFDEF DFS_DELPHI_3}
  N    : Word;
  NuPt : TPoint;
{$ENDIF}
begin
  bFirstTime := True ;
  S := ExtractFilePath(Application.ExeName) +  'Manager.hlp' ;
  if FileExists(S) then
    Application.HelpFile := S
  else
    Application.HelpFile := '';

  {$IFDEF DFS_DELPHI_3}
  MaxPt := Point(0,0);
  for N := 0 to ControlCount-1 do
    with Controls[N].BoundsRect do
    begin
      NuPt := Point(Right + 1 + Width - ClientWidth,
        Bottom + 1 + Height - ClientHeight);
      if NuPt.X > MaxPt.X then MaxPt.X := NuPt.X;
      if NuPt.Y > MaxPt.Y then MaxPt.Y := NuPt.Y;
    end;
  iSignal := 1;
  {$ELSE}
  Constraints.MaxHeight := 409 ;
  Constraints.MaxWidth := 518 ;
  Constraints.MinHeight := 409 ;
  Constraints.MinWidth := 518 ;
  {$ENDIF}

  grdProjects.DataSource := NIL;
  grdProjects.OnDrawDataCell := NIL;
  grdProjects.OnDrawColumnCell := grdProjectsDrawColumnCell;

  // Same as Global.AppPath
  GetDir(0,sPath);

  DragAcceptFiles(Handle,True);
  frmMain.Caption     := 'Project Manager';
  frmMain.borderIcons := [biSystemMenu,biHelp];

  Application.OnHint  := AppOnHint;

end;

procedure TfrmMain.AppOnHint(Sender: TObject);
var T: TComponent ;
begin
  T := Screen.ActiveForm.FindComponent('Statusbar1') ;
  if T <> nil then
    TStatusBar(T).SimpleText := Application.Hint ;
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
var
  Reg : TRegistry ;
begin
  if Table1.Active then
  begin
    Reg := TRegistry.Create ;
    try
      if Reg.OpenKey('Gallagher',True) then
        if Reg.OpenKey('Project Manager', True) then
        begin
          Reg.WriteString('Last Viewed Project',Table1.FieldByName('Name').AsString );
          Reg.CloseKey ;
        end ;
    finally
      Reg.Free ;
    end ;
    Table1.Close ;
  end ;
  DragAcceptFiles(Handle, False);
end;

function TfrmMain.ProjectInQuestion:String;
begin
  if not Table1.Active then exit;

  Result := '';
  with Table1 do
    try
      Result := WhereIsProject + '\' + Trim(MakeFileName);
    except
      Result := '';
    end;
end;

procedure TfrmMain.ToggleCmdStatus(lFlag: Boolean);
var
  i:Integer;
begin
  for i := 1 to ComponentCount -1 do
    if Pos('EXIT',AnsiUpperCase(Components[i].Name)) = 0 then
    begin
      if Components[i] is TButton then
        (Components[i] as TButton).Enabled := lFlag;

      if Components[i] is TMenuItem then
        if (Components[i] as TMenuItem).Tag <> 100 then
          (Components[i] as TMenuItem).Enabled := lFlag;

      if Components[i] is TComboBox then
        (Components[i] as TComboBox).Enabled := lFlag;
    end;
end;

{$IFDEF DFS_DELPHI_3}
procedure TfrmMain.WmGetMinMaxInfo(var Msg: TWmGetMinMaxInfo);
begin
  with Msg.MinMaxInfo^ do
    if iSignal = 1 then
    begin
      with StatusBar1.BoundsRect do
      begin
        PtMinTrackSize.X := Right + 1 + Width - ClientWidth;
        PtMinTrackSize.Y := Bottom + Height - ClientHeight +1;
        PtMaxTrackSize := MaxPt;
      end;
    end;
end;
{$ENDIF}

procedure TfrmMain.mnuDeleteClick(Sender: TObject);
begin
  if not Question('Remove this project?') then
    exit;
  try
    Table1.edit;
    Table1.Delete;
    if Table1.RecordCount = 0 then
      cmdEditProject.Enabled := False;
  except
    ErrMsg('Failed to delete project');
    exit;
  end;
end;

procedure TfrmMain.mnuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.grdProjectsDrawDataCell(Sender: TObject;
  const Rect: TRect; Field: TField; State: TGridDrawState);
begin
  with Sender as TDBGrid, DataSource.DataSet do
  begin
    DrawField(Field.DisplayText,Rect,Canvas,Canvas.Font,Field.Alignment);
  end;
end;

{-----------------------------------------------------------------}
{ Purpose: Change color of a row based on project type            }
{ Comment: May slow traversing the grid.                          }
{-----------------------------------------------------------------}
procedure TfrmMain.grdProjectsDrawColumnCell(Sender: TObject;
  const Rect: TRect; DataCol: Integer; Column: TColumn;
  State: TGridDrawState);
begin
  with Sender as TDBGrid, DataSource.DataSet do
  begin
    DrawField(Column.Field.DisplayText,Rect,Canvas,Column.Font,Column.Alignment);
  end;
end;

{-----------------------------------------------------------------}
{ Purpose: Change color of a row based on project type            }
{-----------------------------------------------------------------}
procedure TfrmMain.DrawField(const Value: String; const Rect: TRect;
   vCanvas: TCanvas; vFont: TFont; vAlignment: TAlignment);
var
  X : Integer;
begin
  X := 0;
  vCanvas.Font := vFont;

  if IsProject16Bit then
    vCanvas.Font.Color := clBlue
  else if not FileExists(ProjectInQuestion) then
    vCanvas.Font.Color := clRed
  else
    exit;

  case vAlignment of
    taRightJustify : begin
      SetTextAlign(vCanvas.Handle,TA_RIGHT);
      X := Rect.Right -2;
    end;
    taLeftJustify  : begin
      SetTextAlign(vCanvas.Handle,TA_LEFT);
      X := Rect.Left +2;
    end;
    taCenter       : begin
      SetTextAlign(vCanvas.Handle,TA_CENTER);
      X := (Rect.Right + Rect.Left) div 2;
    end;
  end;
  vCanvas.TextRect(Rect,X,Rect.Top+2,Value);
  SetTextAlign(vCanvas.Handle,TA_LEFT);
end;

procedure TfrmMain.mnuPackTableClick(Sender: TObject);

begin
  ToggleCmdStatus(False);
  with Table1 do begin
    Active     := False;
    TableType  := ttParadox;
    Exclusive  := True;
    IndexName  := '';
    Open;
    Pack;
  end;

  Information('Packing;Complete');

  with Table1 do
  begin
    Active     := False;
    TableType  := ttParadox;
    Exclusive  := False;
    IndexName  := '';
    Open;
    IndexName := 'ByNameLanguage';
  end;

  OpenProjectTable(False);
  ToggleCmdStatus(True);

  SetScrollPos(grdProjects.Handle,SB_VERT,0,True);
end;

procedure TfrmMain.mnuReindexTableClick(Sender: TObject);
begin
  if Question('Reindex database?') then
  begin
    ToggleCmdStatus(False);
    OpenProjectTable(True);
    with Table1 do
    begin
      Active := True;
      DisableControls;
      try
        RegenerateIndexes;
      finally
        Table1.EnableControls;
        OpenProjectTable(False);
        ToggleCmdStatus(True);
        Information('Reindexing completed!');
      end;
    end;
  end;
end;

{-----------------------------------------------------------------}
{ Purpose:  Open table main in Exclusive or shared mode           }
{-----------------------------------------------------------------}
function TfrmMain.OpenProjectTable(lMode: Boolean): Boolean;
begin

  SetTableProperties(lMode);

  try
    Table1.Open;
  except
    ToggleCmdStatus(False);
    grdProjects.Visible   := False;
    cbChangeIndex.Visible := False;
    cbLangFilter.Visible  := False;
    DBMemo1.Visible       := False;
    dbEditPath.Visible    := False;
    ErrMsg('Unable to open project table');
    Result := False;
    exit;
  end;

  try
    if Table1.IndexDefs.Count = 0 then
      AddIndices;
  except
    ToggleCmdStatus(False);
    grdProjects.Visible   := False;
    cbChangeIndex.Visible := False;
    cbLangFilter.Visible  := False;
    DBMemo1.Visible       := False;
    dbEditPath.Visible    := False;
    ErrMsg('Failed to open projects;Index file(s)');
    Result := False;
    exit;
  end;


  if lMode then
  begin
    Result := True;
    try
      Table1.IndexName := 'ByNameLanguage';
    except
      Result := False;
      exit;
    end;
  end else
  begin
    Result := True;
    try
      Table1.IndexName := 'ByNameLanguage';
    except
      Result := False;
      exit;
    end;
  end;
end;

procedure TfrmMain.wmDropFiles(var Message : TWMDropFiles);
const
  NumOfFilesDroppped = $FFFFFFFF; // Ask count of files dropped on form
var
  DropPt     : TPoint;
  iDropped,I : Integer;
  DroppedName: Array [0..255] of Char;
  S:String;
begin
  DragQueryPoint(Message.Drop,DropPt);
  iDropped := DragQueryFile(Message.Drop,NumOfFilesDroppped,nil,0);

  for I := 0 to iDropped-1 do
  begin
    DragQueryFile(Message.Drop, I, DroppedName, SizeOf(DroppedName));
    S := StrPas(DroppedName);
    { I allow one dropped project, but you might like to
      change that.  Also feel free to enable VB projects }
    if Pos('.DPR',AnsiUpperCaseFileName(S)) >0 then
    begin
      AddDroppedProject(S);
      break
    end;
  end;
  DragFinish(Message.Drop);
end;

procedure TfrmMain.AddDroppedProject(const Info: String);
const  xDesc = 'Place description for ';
var     S:String;
begin
  try
    Table1.Insert;
    Table1.Edit;
    S :=ExtractFileName(Info);
    with Table1 do
    begin
      FieldByName('Name').AsString := xDesc + S + ' here';
      FieldByName('MakeFile').AsString := S;
      FieldByName('Location').AsString := ExtractFileDir(Info);
      FieldByName('Language').AsString := 'Delphi';
      FieldByName('WinType').AsString := '32';
    end;
    Table1.Post;
    grdProjects.UpDate;
    DBMemo1.repaint;
    if Table1.RecordCount > 0 then
      cmdEditProject.Enabled := True;
  except
    MessageDlg('Failed to add dropped project',mtError,[mbOk],0);
    Table1.Cancel;
  end;
  grdProjects.SetFocus;
end;

procedure TfrmMain.mnuIndexClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_TAB, -2) ;
end;

procedure TfrmMain.mnuAboutClick(Sender: TObject);
var
  f:TMyAboutForm ;
begin
  {ShellAbout(Self.Handle,PChar('Project Manager'),'By Kevin S Gallagher',
    Application.Icon.Handle) ;}
  f := TMyAboutForm.Create(Self) ;
  f.CenterOnParent := True ;
  f.Title := 'Project Manager' ;
  f.execute ;
end;

procedure TfrmMain.mnuFindClick(Sender: TObject);
begin
  { Runs Windows Find dialog }
  ShellExecute(Application.Handle, 'find', nil{'c:\'},nil,nil,SW_SHOWNORMAL) ;
end;

procedure TfrmMain.ViewknownDPKfile1Click(Sender: TObject);
var
  f:TfrmReadDpkFiles ;
begin
   f:= TfrmReadDpkFiles.Create(Self) ;
   try
    CenterForm(f) ;
    f.ShowModal ;
   finally
    f.Free ;
   end ;
end;

procedure TfrmMain.mnuCodeInSightClick(Sender: TObject);
var
  f: TfrmInSight ;
begin
  f := TfrmInSight.Create(Self) ;
  try
    CenterForm(f) ;
    f.ShowModal ;
  finally
    f.Free ;
  end ;
end;

procedure TfrmMain.mnuDelphiDirectClick(Sender: TObject);
var
  f: TfrmDelphiDirect ;
begin
  f := TfrmDelphiDirect.Create(Self) ;
  try
    CenterForm(f) ;
    f.ShowModal ;
  finally
    f.Free ;
  end ;
end;

end.
