unit ReopenProjectList;
{------------------------------------------------------------------------------
  Description
    This form provides you with easy access to
    1)  Tweak the "Closed Project" list for Delphi projects.
          - Reorder the list from the current project in the list.
          - Reorder the list by adding a new project to the list.
    2)  Setting the project to open in Delphi the next time Delphi is used.
    3)  Remove projects from the "Closed Project" key in the registry.
  Author
    Kevin S. Gallagher  gallaghe@teleport.com

  Requires
    Delphi 4 or greater (may work with D3) along with DPRegstr.pas which
    is supplied (along with JCL.INC).

  Comments
    + The values under the "ClosedProject" key have an undocumented structure.
      There are 3 sections to each value,
        a) Module type
        b) project path and project file
        c) bunch of numbers
      My code will work fine in most cases, it may write wrong information
      to section (a) and (c) but should not interfer with opening projects. If
      you happen to come across with all known values for section (a) and (c)
      I'd like to know.
    + This is the only release for this other unless I get meaning to the above.

  provided free of charge (not public domain)

  NOT TO BE SOLD OR DISTRIBUTED FOR A CHARGE without my permission.

  No warranty is implied and we are not responsible for any problems that might
  arise from the use of this unit. You use this unit entirely at your own risk.
 -----------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ComCtrls, ImgList, ExtCtrls, Menus;

type
  TfrmProjectListing = class(TForm)
    ListBox1: TListBox;
    StatusBar1: TStatusBar;
    cmdMoveUp: TBitBtn;
    cmdMoveDown: TBitBtn;
    cmdExit: TButton;
    cmdSetProjectOrder: TButton;
    cmdSetProject: TButton;
    PopupMenu1: TPopupMenu;
    pMnuFromList: TMenuItem;
    pMnuUserSelected: TMenuItem;
    OpenDialog1: TOpenDialog;
    pMnuFlushmostrecentproject: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure cmdMoveUpClick(Sender: TObject);
    procedure cmdMoveDownClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ListBox1DrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure FormDestroy(Sender: TObject);
    procedure cmdExitClick(Sender: TObject);
    procedure cmdSetProjectOrderClick(Sender: TObject);
    procedure cmdSetProjectClick(Sender: TObject);
    procedure pMnuFromListClick(Sender: TObject);
    procedure pMnuUserSelectedClick(Sender: TObject);
    procedure pMnuFlushmostrecentprojectClick(Sender: TObject);
    procedure ListBox1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    cLastSession:String ;
    imgArrow: TBitMap ;
    bFirst: Boolean ;
    procedure DeleteHilightedProject ;
  public
    { Public declarations }
  end;

var
  frmProjectListing: TfrmProjectListing;

implementation

uses DPRegstr, registry, filectrl, ShellApi;

{$R *.DFM}
{$R Extras.res}

  function Question(const S: String):Boolean;
  begin
    case MessageDlg(S,mtConfirmation,[mbYes,mbNo],0) of
       IDYES    : Result := True;
       IDNO     : Result := False;
    else          Result := False;
    end;
  end;

{------------------------------------------------------------------------------
  Populates ListBox1 with all "File->Reopen" projects, sets up the bitmap used
  to point out the "next" project to open in Delphi.
 -----------------------------------------------------------------------------}
procedure TfrmProjectListing.FormCreate(Sender: TObject);
var
  Reg: TRegistry ;
  cKey: String ;
  cTmp: String ;
  cName: String ;
  icoIndex: word ;
  iPos: Integer ;
begin
  bFirst := True ;
  try
    cLastSession := DelphiLastSession ;
    ListBox1.Items.AddStrings( DelphiProjects ) ;
    if ListBox1.Items.Count = -1 then
    begin
      cmdSetProjectOrder.Enabled := False ;
      exit ;
    end ;

    ListBox1.ItemIndex := 0 ;
    ListBox1.Refresh ;
  except
    ListBox1.Clear ;
  end ;

  imgArrow := TBitMap.Create ;
  imgArrow.Handle := LoadBitMap(hInstance, 'LASTPROJECT') ;
  if imgArrow.Handle = 0 then begin
    imgArrow.Free ;
    imgArrow := nil ;
    MessageDlg('Last Project bitmap from resources',mtError,[mbOk],0) ;
  end ;

  Caption := 'Recent Delphi ' +DelphiVersionStr+ ' project list' ;
  Application.Title := Caption ;
  
  Reg := TRegistry.Create ;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT ;
    if Reg.OpenKeyReadOnly('.dpr') then
    begin
      cKey := Reg.ReadString('') ;
      Reg.CloseKey ;
      if Reg.KeyExists(cKey + '\DefaultIcon') then
        if Reg.OpenKeyReadOnly(cKey + '\DefaultIcon') then
        begin
          cTmp := Trim(Reg.ReadString('')) ;
          Reg.CloseKey ;
        end ;
    end ;
  finally
    Reg.Free ;
  end ;

  if Length(cTmp) = 0 then exit ;

  iPos := Pos(',',cTmp) ;
  cName := Copy(cTmp,1,iPos -1) ;
  icoIndex := StrToIntDef(Trim(Copy(cTmp,iPos +1,Length(cTmp) - iPos)),1) ;
  try
    Application.Icon.Handle := ExtractAssociatedIcon(hInstance,PChar(cName),icoIndex) ;
  except
  end ;
end;

{------------------------------------------------------------------------------
  Used to free up resources created by us.
 -----------------------------------------------------------------------------}
procedure TfrmProjectListing.FormDestroy(Sender: TObject);
var
  i,iCount: Integer ;
begin
  if imgArrow <> nil then  imgArrow.Free ;

  iCount := ListBox1.Items.Count ;
  for i := 0 to iCount -1 do
    if ListBox1.Items.Objects[i] <> nil then ListBox1.Items.Objects[i].Free ;
end;

procedure TfrmProjectListing.cmdExitClick(Sender: TObject);
begin
  Close ;
end;

procedure TfrmProjectListing.cmdMoveUpClick(Sender: TObject);
var
  iItem: Integer ;
begin
  iItem := ListBox1.ItemIndex ;

  if iItem = -1   then exit ;
  if iItem -1 < 0 then exit ;

  with ListBox1 do
  begin
    Items.Move(iItem, iItem -1) ;
    ItemIndex := iItem -1 ;
    SetFocus ;
  end ;

end;

procedure TfrmProjectListing.cmdMoveDownClick(Sender: TObject);
var
  iItem: Integer ;
begin
  iItem := ListBox1.ItemIndex ;
  if iItem = -1 then exit ;
  if iItem +1 > ListBox1.Items.Count -1 then exit ;

  with ListBox1 do
  begin
    Items.Move(iItem, iItem +1) ;
    ItemIndex := iItem +1 ;
    SetFocus ;
  end ;

end;

{------------------------------------------------------------------------------
  Rather then making this a non-sizeable form with a horizontal scroll-bar on
  the ListBox I elected to make the form sizeable. Doing so calls for code to
  resize the width of the ListBox and to make all look nice there is code to
  reposition the buttons. Be sure to note that this code is not attempting
  to win any prizes.
 -----------------------------------------------------------------------------}
procedure TfrmProjectListing.FormResize(Sender: TObject);
var
  cw : Integer ;
  bw : Integer ;
begin

  cw := ClientWidth ;
  bw := cmdMoveUp.Width ;

  ListBox1.Width := (cw - bw) - 15 ;

  cmdMoveUp.Left := (cw - bw) - 5;
  cmdMoveDown.Left := cmdMoveUp.Left ;

  bw := cmdExit.Width ;
  cmdExit.Left := (cw - bw) - 5 ;
  cmdSetProjectOrder.Left := (cw - (bw *2)) - 9 ;

  cmdExit.Top := (ClientHeight - StatusBar1.Height)  - cmdExit.Height - 2 ;
  cmdSetProjectOrder.Top := cmdExit.Top ;

  cmdSetProject.Top := cmdExit.Top ;
  cmdSetProject.Left := (cw - (cmdSetProjectOrder.Width *3)) - 14 ;

  if bFirst then
  begin
    cmdExit.Visible := True ;
    cmdSetProjectOrder.Visible := True ;
    cmdSetProject.Visible := True ;
    bFirst := False ;
  end ;

end;

{------------------------------------------------------------------------------
  Used to draw bitmap for showing "next" project Delphi will open.
 -----------------------------------------------------------------------------}
procedure TfrmProjectListing.ListBox1DrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
begin
  with Control as TListBox, Canvas do
  begin
    FillRect(Rect) ;
    try
      try
        if CompareText(Items.Strings[Index], cLastSession) =0 then
        begin
          StretchDraw(Bounds(Rect.Left+2,Rect.Top,imgArrow.Width,ItemHeight-2),
            imgArrow) ;
        end ;
      except
        on Exception do  TextRect(Rect, Rect.Left+2, Rect.Left+2,'ERROR') ;
      end ;
    finally
      Rect.Left := Rect.Left + 16 ;
      TextRect(Rect, Rect.Left+2, Rect.Top, Items[Index]) ;
    end ;
  end ;
end;

{------------------------------------------------------------------------------
  Method for reordering Delphi's "File->Reopen" list
 -----------------------------------------------------------------------------}
procedure TfrmProjectListing.cmdSetProjectOrderClick(Sender: TObject);
var
  Reg   : TRegistry ;
  cItem : String ;
  S     : String ;
  i,
  iCount: Integer ;
begin
  if ListBox1.Items.Count = 0 then
  begin
    DeleteClosedProjects ;
    exit ;
  end ;

  { Remove all values from the open key }
  if not DeleteClosedProjects then
    exit ;

  Reg := TRegistry.Create ;
  try
    if Reg.OpenKey(Format(ProjectsKey,[DelphiVersionStr]),False) then
    begin
      iCount := ListBox1.Items.Count ;
      if iCount > 5 then
        iCount := 5 ;

      for i := 0 to iCount -1 do
      begin
        S := ListBox1.Items.Strings[i] ;
        cItem := 'File_' + IntToStr(i) ;
        with ListBox1 do
          with Items.Objects[i] as TSessionObj do
            Reg.WriteString(cItem,cParam1 + #44#39 + S + #39#44 + cParam2) ;
      end ;
      Reg.CloseKey ;
    end ;
  finally
    Reg.Free ;
  end ;
end;

procedure TfrmProjectListing.cmdSetProjectClick(Sender: TObject);
var
  P: TPoint;
begin
  P := ClientToScreen(Point(cmdSetProject.left, cmdSetProject.Top + cmdSetProject.Height)) ;
  PopUpMenu1.Popup(P.x , P.y ) ;
end;

{------------------------------------------------------------------------------
  Provides method to set the next file to open when Delphi is fired up.
 -----------------------------------------------------------------------------}
procedure TfrmProjectListing.pMnuFromListClick(Sender: TObject);
var S: String ;
begin
  if ListBox1.Items.Count = 0 then
  begin
    DeleteClosedProjects ;
    exit ;
  end ;

  if ListBox1.ItemIndex = -1 then
    exit ;
    
  S := ListBox1.Items.Strings[ListBox1.ItemIndex] ;
  if SetSession(S) then
  begin
    cLastSession := S ;
    ListBox1.Refresh ;
  end else MessageDlg('Operation failed',mtError,[mbOk],0) ;
end;

{------------------------------------------------------------------------------
  Description:
  Selecting "User Select" from the popup menu permits you to select an existing
  project (group file) to open the next time Delphi starts.

  Comments:
  The function SetSession does the job but does not place this project into the
  CloseProject key in the registry. Most of the code above it is for placing
  the selected project into the CloseProject key. I am not totally sure that
  the pieces are correct. To be specific, the key is broken down into several
  parts which are undocumented.

  Sections
  1. SourceModule
  2. Project
  3. Bunch of digits

  I am positive that section 2 is correct, sort of on part 1, never seen it
  anything else. Very unsure on section 3.

  So I suggest that before using the code, investigate you values prior to
  using this code.

  Suggestion:
  Use RegEdit to export the key below, replace 4.0 with your version of Delphi.
  HKEY_CURRENT_USER\Software\Borland\Delphi\x.0\Closed Projects

  NOTE: This code is called from ListBox1.KeyDown, hence do not use "Sender"
 -----------------------------------------------------------------------------}
procedure TfrmProjectListing.pMnuUserSelectedClick(Sender: TObject);
var
  Reg   : TRegistry ;
  cPath : String ;
  cItem : String ;
  S     : String ;
  i,
  iCount: Integer ;
begin
  Reg := TRegistry.Create ;
  try
    if Reg.OpenKeyReadOnly('Gallagher') then
      if Reg.OpenKeyReadOnly('Project Manager') then
        cPath := Trim(Reg.ReadString('Last Browse for Project Path')) ;
    Reg.CloseKey ;
  finally
    Reg.Free ;
  end ;

  if Length(cPath) >0 then
    if DirectoryExists(cPath) then
      OpenDialog1.InitialDir := cPath ;

  if not OpenDialog1.Execute then
    exit ;

  { not sure about SourceModule and 1,1,1,1,1,0,1 - not documented by Inprise }
  ListBox1.Items.InsertObject(0,OpenDialog1.FileName,
    TSessionObj.Create('SourceModule','1,1,1,1,1,0,1')) ;

  i := ListBox1.Items.Count -1 ;
  if ListBox1.Items.Objects[i] <> nil then ListBox1.Items.Objects[i].Free ;
  ListBox1.Items.Delete(i) ;
  if ListBox1.Items.Count = -1 then
    exit ;

  { Remove all values from the open key }
  if not DeleteClosedProjects then
    exit ;

  Reg := TRegistry.Create ;
  try
    if Reg.OpenKey(Format(ProjectsKey,[DelphiVersionStr]),False) then
    begin
      iCount := ListBox1.Items.Count ;
      if iCount > 5 then
        iCount := 5 ;

      for i := 0 to iCount -1 do
      begin
        S := ListBox1.Items.Strings[i] ;
        cItem := 'File_' + IntToStr(i) ;
        with ListBox1 do
          with Items.Objects[i] as TSessionObj do
            Reg.WriteString(cItem,cParam1 + #44#39 + S + #39#44 + cParam2) ;
      end ;
      Reg.CloseKey ;
    end ;
  finally
    Reg.Free ;
  end ;

  if SetSession(OpenDialog1.FileName) then
  begin
    cLastSession := OpenDialog1.FileName ;
    ListBox1.ItemIndex := 0 ;
    ListBox1.Refresh ;
  end else MessageDlg('Operation failed',mtError,[mbOk],0) ;

  cPath := ExtractFilePath(OpenDialog1.FileName) ;

  Reg := TRegistry.Create ;
  try
    if Reg.OpenKey('Gallagher',True) then
      if Reg.OpenKey('Project Manager',True) then
        Reg.WriteString('Last Browse for Project Path',cPath) ;
    Reg.CloseKey ;
  finally
    Reg.Free ;
  end ;
end;

{------------------------------------------------------------------------------
  Description:  Used to flush "next" project to open when Delphi starts up next
     Comments:  This file does not have to be in the "Reopen" list.
 ------------------------------------------------------------------------------}
procedure TfrmProjectListing.pMnuFlushmostrecentprojectClick(
  Sender: TObject);
begin
  if Question('Flush ' + cLastSession + ' ?') then
    if FlushProject then
    begin
      cLastSession := '' ;
      ListBox1.Refresh ;
    end else
      MessageDlg('Failed to flush ' + cLastSession,mtError,[mbOk],0) ;

end;

{------------------------------------------------------------------------------
  Provides method to remove an item from the "ClosedProjects" key. This code
  only removes the project from the ListBox, you need to execute the button
  "Set Order" to save the information to the registry.
 ------------------------------------------------------------------------------}
procedure TfrmProjectListing.DeleteHilightedProject ;
var
  iItem: Integer ;
begin
  if not Question('Remove all projects from "ClosedProjects" key?') then
    exit ;
    
  if ListBox1.ItemIndex = -1 then
    exit ;
  try
    iItem := ListBox1.ItemIndex ;
    if ListBox1.Items.Objects[iItem] <> nil then
      ListBox1.Items.Objects[iItem].Free ;
    ListBox1.Items.Delete(iItem) ;
    Dec(iItem) ;
    if iItem <> -1 then
      ListBox1.ItemIndex := iItem ;
  except
    MessageDlg('Failed to remove project from list',mtError,[mbOk],0) ;
  end ;
end ;

procedure TfrmProjectListing.ListBox1KeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_DELETE: DeleteHilightedProject ;
    VK_INSERT: pMnuUserSelectedClick(Self) ;
  end ;
end;

end.
