{***********************************************************************}
{ Description:  Project Manager                                         }
{  Programmer:  Kevin S Gallagher     gallaghe@teleport.com             }
{    Requires:  TBrkApart component (FreeWare)                          }
{    Versions:  4 and 5 of Delphi                                       }
{                                                                       }
{ KSG 11/09/1997  Fixed minor error in procedure btnGetPathClick that   }
{                 called TableCode.WhereIsProject which used .AsString  }
{                 in FieldByName rather then .Value which may seem      }
{                 wrong, but makes my logic work.                       }
{ KSG 03/18/2000  Changes [ProjectTypeToUse] logic.                     }
{***********************************************************************}

unit frmAddTo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, db,DBCtrls, FileCtrl, Mask, ComCtrls;


type
  TfrmEdit = class(TForm)
    lblProject: TLabel;
    edtProject: TDBEdit;
    lblPath: TLabel;
    dbePath: TDBEdit;
    lblMakeFile: TLabel;
    dbeMakeFile: TDBEdit;
    lblPlatform: TLabel;
    dbePlatform: TDBEdit;
    lblLanguage: TLabel;
    dbeLanguage: TDBEdit;
    lblNotes: TLabel;
    DBMemo1: TDBMemo;
    btnExit: TButton;
    btnGetPath: TButton;
    OpenDialog1: TOpenDialog;
    btnMakeFile: TButton;
    btnCancel: TButton;
    StatusBar1: TStatusBar;
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnGetPathClick(Sender: TObject);
    procedure btnMakeFileClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    function ProjectTypeToUse(const cPath: string): Integer;
    procedure DataIsDirty(Sender: TObject);
    procedure DoDataChangeEvents;
  public
    { Public declarations }
  end;

var
  frmEdit: TfrmEdit;
  isFlag : Integer;
  { Refer to btnGetPathClick event for how this array is used. You need
    to change the paths to suit existing directories on your computer. }
  aPaths : array[0..3] of string =
    ('C:\Program Files\Borland\Projects','D:\Code Storage',
     'C:\Work','C:\Delphi\Projects');
     
implementation

uses Main, Global, TableCode;

{$R *.DFM}

{-----------------------------------------------------------------}
{ Purpose:  To toggle filter in a dialog box                      }
{-----------------------------------------------------------------}
function TfrmEdit.ProjectTypeToUse(const cPath: string): Integer;
const
  ProjType : PChar = 'DPR'#13'BPG'#13'VBP'#13 ;
var
  Rec   : TSearchRec ;
  Res   : Integer ;
  cExt  : String ;
  aList : TStringList ;
  iPos  : Integer ;
begin
  Result := 1 ;
  FillChar(Rec,SizeOf(TSearchRec),0) ;
  Res := FindFirst(cPath + '\*.*', faAnyFile, Rec) ;
  if Res = 0 then
  begin
    aList := TStringList.Create ;
    try
      aList.SetText(ProjType) ;
      while Res = 0 do
      begin
        with Rec do
        begin
          if Rec.Name[1] <> '.' then
          begin
            cExt := UpperCase(ExtractFileExt(Rec.Name)) ;
            Delete(cExt,1,1) ;
            iPos := aList.IndexOf(cExt) ;
            if iPos > -1 then
            begin
              Result := iPos ;
              break ;
            end ;
          end ;
        end ;
        Res := FindNext(Rec) ;
      end ;
    finally
      aList.Free ;
    end ;
    FindClose(Rec) ;
  end ;
end;

procedure TfrmEdit.FormActivate(Sender: TObject);
begin
  isFlag := 0;

  if frmMain.Tag = ADD_NEW_PROJECTS then
  begin
    frmMain.Table1.Insert;
    Self.Caption      := 'Inserting new project';
    btnExit.Caption   := '&Add';
    btnCancel.Visible := True;
  end;
  if frmMain.Tag = EDIT_CURRENT_PROJECT then
  begin
    frmMain.Table1.Edit;
    Self.Caption      := 'Editing';
    btnExit.Caption   := '&Save';
    btnCancel.Visible := False;
  end;

  edtProject.DataField := 'Name';
  dbePath.DataField    := 'Location';
  dbeMakeFile.DataField:= 'MakeFile';
  dbePlatform.DataField:= 'WinType';
  dbeLanguage.DataField:= 'Language';
  DBMemo1.DataField    := 'Notes';

  frmMain.Table1.Edit;
  DoDataChangeEvents;

  edtProject.SetFocus;
end;

procedure TfrmEdit.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  frmMain.grdProjects.UpDate;
end;

procedure TfrmEdit.btnExitClick(Sender: TObject);
// Table1.State in [dsEdit,dsInsert]
begin
  if frmMain.Tag = EDIT_CURRENT_PROJECT then
  begin
    if Self.Tag > 0 then
    begin
      if Question('Save changes?') then
      begin
        frmMain.Table1.Edit;
        frmMain.Table1.Post;
        isFlag := 1;
        Close;
        exit;
      end else
      begin
        frmMain.Table1.Edit;
        frmMain.Table1.Cancel;
        isFlag := 1;
        Close;
        exit;
      end;
    end;
  end;

  { Code for adding new projects }
  with frmMain do
  begin
    Table1.Edit;
    Table1.Post;
    grdProjects.UpDate;
    DBMemo1.repaint;
  end;

  if frmMain.Table1.RecordCount > 0 then
  begin
    frmMain.cmdEditProject.Enabled  := True;
    frmMain.mnuDelete.Enabled       := True;
    frmMain.mnuRunProject.Enabled   := True;
  end;
  isFlag := 1;                           { see FormCloseQuery  }
  Close;
end;

procedure TfrmEdit.FormCreate(Sender: TObject);
begin
  Self.BorderIcons := [biSystemMenu,biHelp];
end;

procedure TfrmEdit.btnGetPathClick(Sender: TObject);
var
  i:Integer;
  cPath: String;
begin

  try
    cPath := Trim(WhereIsProject);
  except
    cPath := 'C:\';
    for i := Low(aPaths) to High(aPaths) do
    begin
      if DirectoryExists(aPaths[i]) then
      begin
        cPath := aPaths[i];
        BREAK;
      end;
    end;
  end;

  if Length(cPath) = 3 then
    SetCurrentDirectory(PChar(cPath));

  if SelectDirectory(cPath,[],0) then
  begin
    frmMain.Table1.Edit;
    frmMain.Table1.FieldByName('Location').AsString := cPath;
    if Length(Trim(dbeMakeFile.EditText)) > 0 then
    begin
      dbePath.SetFocus;
      dbePath.SelStart := 0;
    end else
      dbeMakeFile.SetFocus;
  end;
end;

procedure TfrmEdit.btnMakeFileClick(Sender: TObject);
var
  cPath:String;
  cName:String;
begin
  try
    cPath := Trim(frmMain.Table1.FieldByName('Location').AsString);
  except
    ErrMsg('Location must be selected first');
    exit;
  end;

  if length(cPath) > 0 then
  begin
    OpenDialog1.InitialDir := cPath;
  end;

  OpenDialog1.Filter      := PrjFlt;
  OpenDialog1.FilterIndex := ProjectTypeToUse(cPath);

  if OpenDialog1.Execute then
  begin
    cName := ExtractFileName(OpenDialog1.FileName);
    frmMain.Table1.Edit;
    frmMain.Table1.FieldByName('MakeFile').AsString := cName;
    frmMain.Table1.FieldByName('WinType').AsString  := '32';
    dbePlatform.EditText := '32';
    if UpperCase(ExtractFileExt(cName)) = '.DPR' then
    begin
      dbeLanguage.EditText := 'Delphi';
      frmMain.Table1.FieldByName('Language').AsString := 'Delphi';
    end;
    if UpperCase(ExtractFileExt(cName)) = '.VBP' then
    begin
      dbeLanguage.EditText := 'VB';
      frmMain.Table1.FieldByName('Language').AsString := 'VB';
    end;
    DBMemo1.SetFocus;
  end else
    dbeMakeFile.SetFocus;
  {}

  cPath := '';
  cName := '';
end;

procedure TfrmEdit.btnCancelClick(Sender: TObject);
begin
  frmMain.Tag := ABORT_ADDING_RECORD;
  if frmMain.Table1.RecordCount > 0 then
  begin
    try
      frmMain.Table1.Edit;
      frmMain.Table1.Cancel;
    except
      ErrMsg('Failed aborting adding new project!!!');
    end;
  end;
  isFlag := 1;
  Close;
end;

procedure TfrmEdit.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if isFlag = 0 then
  begin
    Beep;
    CanClose := False;
  end;
end;

procedure TfrmEdit.DataIsDirty(Sender: TObject);
begin
  if Self.Visible then
  begin
    Self.Tag := 1;
  end;
end;

procedure TfrmEdit.DoDataChangeEvents;
var
  i:Integer;
begin
  if frmMain.Tag = ADD_NEW_PROJECTS then
    exit;

  for i := 1 to ComponentCount -1 do
    if Components[i] is TDBEdit then
      (Components[i] as TDBEdit).OnChange := DataIsDirty;

end;
end.
